@@ -18,12 +18,20 @@
 from google.cloud.firestore_v1 import __version__
 from google.cloud.firestore_v1 import ArrayRemove
 from google.cloud.firestore_v1 import ArrayUnion
+from google.cloud.firestore_v1 import AsyncClient
+from google.cloud.firestore_v1 import AsyncCollectionReference
+from google.cloud.firestore_v1 import AsyncDocumentReference
+from google.cloud.firestore_v1 import AsyncQuery
+from google.cloud.firestore_v1 import async_transactional
+from google.cloud.firestore_v1 import AsyncTransaction
+from google.cloud.firestore_v1 import AsyncWriteBatch
 from google.cloud.firestore_v1 import Client
+from google.cloud.firestore_v1 import CollectionGroup
 from google.cloud.firestore_v1 import CollectionReference
 from google.cloud.firestore_v1 import DELETE_FIELD
 from google.cloud.firestore_v1 import DocumentReference
 from google.cloud.firestore_v1 import DocumentSnapshot
-from google.cloud.firestore_v1 import enums
+from google.cloud.firestore_v1 import DocumentTransform
 from google.cloud.firestore_v1 import ExistsOption
 from google.cloud.firestore_v1 import GeoPoint
 from google.cloud.firestore_v1 import Increment
@@ -39,18 +47,27 @@ from google.cloud.firestore_v1 import types
 from google.cloud.firestore_v1 import Watch
 from google.cloud.firestore_v1 import WriteBatch
 from google.cloud.firestore_v1 import WriteOption
+from typing import List
 
 
-__all__ = [
+__all__: List[str] = [
     "__version__",
     "ArrayRemove",
     "ArrayUnion",
+    "AsyncClient",
+    "AsyncCollectionReference",
+    "AsyncDocumentReference",
+    "AsyncQuery",
+    "async_transactional",
+    "AsyncTransaction",
+    "AsyncWriteBatch",
     "Client",
+    "CollectionGroup",
     "CollectionReference",
     "DELETE_FIELD",
     "DocumentReference",
     "DocumentSnapshot",
-    "enums",
+    "DocumentTransform",
     "ExistsOption",
     "GeoPoint",
     "Increment",
